@@ -43,6 +43,8 @@ else:
 
 _C = TypeVar("_C", bound="Cursor[Any]")
 
+ACTIVE = pq.TransactionStatus.ACTIVE
+
 
 class BaseCursor(Generic[ConnectionType, Row]):
     # Slots with __weakref__ and generic bases don't work on Py 3.6
@@ -281,9 +283,9 @@ class BaseCursor(Generic[ConnectionType, Row]):
         self._execute_send(pgq, binary=binary, no_pqexec=True)
         self._pgconn.set_single_row_mode()
         self._last_query = query
+        yield from send(self._pgconn)
 
     def _stream_fetchone_gen(self, first: bool) -> PQGen[Optional["PGresult"]]:
-        yield from send(self._pgconn)
         res = yield from fetch(self._pgconn)
         if res is None:
             return None
@@ -575,14 +577,25 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
         """
         Iterate row-by-row on a result from the database.
         """
-        with self._conn.lock:
-            self._conn.wait(self._stream_send_gen(query, params, binary=binary))
-            first = True
-            while self._conn.wait(self._stream_fetchone_gen(first)):
-                rec = self._tx.load_row(0, self._make_row)
-                assert rec is not None
-                yield rec
-                first = False
+        try:
+            with self._conn.lock:
+                self._conn.wait(self._stream_send_gen(query, params, binary=binary))
+                first = True
+                while self._conn.wait(self._stream_fetchone_gen(first)):
+                    # We know that, if we got a result, it has a single row.
+                    rec: Row = self._tx.load_row(0, self._make_row)  # type: ignore
+                    yield rec
+                    first = False
+        except e.Error as ex:
+            # try to get out of ACTIVE state. Just do a single attempt, which
+            # shoud work to recover from an error or query cancelled.
+            if self._pgconn.transaction_status == ACTIVE:
+                try:
+                    self._conn.wait(self._stream_fetchone_gen(first))
+                except Exception:
+                    pass
+
+            raise ex.with_traceback(None)
 
     def fetchone(self) -> Optional[Row]:
         """
@@ -664,8 +677,11 @@ class Cursor(BaseCursor["Connection[Any]", Row]):
 
         :rtype: Copy
         """
-        with self._conn.lock:
-            self._conn.wait(self._start_copy_gen(statement))
+        try:
+            with self._conn.lock:
+                self._conn.wait(self._start_copy_gen(statement))
 
-        with Copy(self) as copy:
-            yield copy
+            with Copy(self) as copy:
+                yield copy
+        except e.Error as ex:
+            raise ex.with_traceback(None)
